#ifndef _TOOLS_H_
#define _TOOLS_H_

// rozne pomocne nastroje **************************************************

// makra pre cykly
#define loop(v,m) for (int v=0;v<(int) (m);v++)
#define loopi(m) loop(i,m)
#define loopj(m) loop(j,m)
#define loopk(m) loop(k,m)
#define loopl(m) loop(l,m)

#define loopvi(v) for (int i=0;i<(v).length();i++)
#define loopvj(v) for (int j=0;j<(v).length();j++)
#define loopvk(v) for (int k=0;k<(v).length();k++)
#define loopvl(v) for (int l=0;l<(v).length();l++)

// makra pre rusenie ukazovatelov
#define DELETEP(p) if (p) { delete   p; p=NULL; }
#define DELETEA(p) if (p) { delete[] p; p=NULL; }

// 1-rozmerny udajovy typ - vektor *****************************************
template <class T>
struct vector
{
 // udaje ------------------------------------------------------------------
 T *buf;    // pamatove miesto pre ulozenie prvkov vektora
 unsigned char *newbuf; // pomocny ukazovatel
 int vlen;  // celkovy pocet prvkov vektora
 int mlen;  // pocet prvkov vektora, pre ktore je vyhradene pamatove miesto
 int plen;  // pomocna premenna

 // konstruktory -----------------------------------------------------------
 vector():buf(NULL),vlen(0),mlen(0),plen(0) {}
 vector(const vector &V):buf(NULL),vlen(0),mlen(0),plen(0) { *this=V; }

 // destruktor -------------------------------------------------------------
 ~vector() {
   shrink(0);
   if (buf) delete[] (unsigned char *) buf; buf=NULL;
   vlen=0; mlen=0; plen=0;
  }

 // operatory --------------------------------------------------------------
 // priradenie
 vector<T> &operator=(const vector<T> &V) {
  shrink(0);
  if (V.length()>mlen) vrealloc(V.length());
  loopvi(V) add(V[i]);
  return(*this);
 }
 // index
 T &operator[](int I) { if (I>=0 && I<vlen) return(buf[I]); else return(buf[0]); }
 const T &operator[](int I) const { if (I>=0 && I<vlen) return(buf[I]); else return(buf[0]); }

 // funkcie ----------------------------------------------------------------
 // zvacsenie pamatoveho miesta vyhradeneho pre prvky vektora
 void vrealloc(int I) {
  // pocet prvkov, pre ktore je uz vyhradene pamatove miesto
  plen=mlen;
  // ak este nebole vyhradene pamatove miesto, vyhradi sa miesto pre I prvkov (min. 8)
  // v opacnom pripade sa vyhradene pamatove miesto postupne zvacsuje 2x az pokial
  // nebude dosiahnuta potrebna velkost
  if (!mlen) mlen=(8 > I ? 8 : I);
  else while (mlen<I) mlen*=2;
  // ak nie je potrebne vyhradit nove pamatove miesto
  if (mlen<=plen) return;
  // vyhradenie noveho pamatoveho miesta pre mlen prvkov
  newbuf=new unsigned char[mlen*sizeof(T)];
  // kopia obsahu povodneho pamatoveho miesta do noveho pamatoveho miesta
  if (plen>0) {
   memcpy(newbuf,buf,plen*sizeof(T));
   delete[] (unsigned char *) buf; // zrusenie povodneho pamatoveho miesta
  }
  buf=(T *) newbuf;
 }
 // pridanie prvku vektora
 T &add() {
  if (vlen==mlen) vrealloc(vlen+1);
  // 'placement new operator' umiestni objekt do predpripraveneho pamatoveho miesta,
  // spusti sa konstruktor objektu
  new (&buf[vlen]) T();
  return(buf[vlen++]);
 }
 T &add(const T &X) {
  if (vlen==mlen) vrealloc(vlen+1);
  new (&buf[vlen]) T(X);
  return(buf[vlen++]);
 }
 // duplikacia posledneho prvku vektora
 T &dup() {
  if (vlen==mlen) vrealloc(vlen+1);
  new (&buf[vlen]) T(buf[vlen-1]);
  return(buf[vlen++]);
 }
 // pocet prvkov vektora
 int length() const { return(vlen); }
 // velkost vyhradenej pamate
 int capacity() const { return(mlen); }
 // je pocet prvkov vektora nulovy?
 bool isempty() const { return(vlen==0); }
 // je pocet prvkov vektora v rozsahu?
 bool inrange(size_t I) const { return(I<size_t(vlen)); }
 bool inrange(int I) const { return(I>=0 && I<vlen); }
 // zrusenie posledneho prvku vektora
 // z dovodu pouzitia 'placement new operatora' je nutne explicitne spustit destruktor objektu
 void destruct() { buf[--vlen].~T(); }
 // zmensenie poctu prvkov vektora
 void resize(int I)         { vrealloc(I); vlen=I; }
 void shrink(int I)         { while (vlen>I) destruct(); }
 void shrinknodelete(int I) { vlen=I; }
 // reset vektora
 void reset() {
  shrink(0); DELETEA(buf); buf=NULL; vlen=0; mlen=0; plen=0;
 }
 // ziskanie posledneho prvku vektora, pocet prvkov sa zmensi
 T &pop() { return(buf[--vlen]); }
 // ziskanie posledneho prvku vektora, pocet prvkov sa nezmeni
 T &last() { return(buf[vlen-1]); }
 // ziskanie adresy pamatoveho miesta pre ulozenie prvkov vektora
 T *getbuf() { return(buf); }
 const T *getbuf() const { return(buf); }
 // v pripade, ze prvok vektora je ukazovatelom na pamatove miesto,
 // najskor je nutne uvolnit toto miesto, potom vyvolat destruktor
 // pre prvok vektora a nakoniec zrusit samotny vektor
 void deletecontentsp() { while (!isempty()) delete   pop(); }
 void deletecontentsa() { while (!isempty()) delete[] pop(); }
 // nahradenie urcevch prvkov vektora posledvm prvkom
 void replacewithlast(const T &O) {
  if (!vlen) return;
  loopi(vlen-1) if (buf[i]==O) { buf[i]=buf[vlen-1]; }
  vlen--;
 }
 // vymena prvkov 1. a 2. polovice vektora
 void reverse() {
  loopi(vlen/2) swap(buf[i],buf[vlen-1-i]);
 }
 // vlozenie noveho prvku vektora na poziciu I
 T &insert(int I,const T &P) {
  add(T());
  for (int j=vlen-1;j>I;j--) buf[j]=buf[j-1];
  buf[I]=P;
  return(buf[I]);
 }
 // vlozenie N novych prvkov vektora na poziciu I
 T *insert(int I,const T *P,int N) {
  if (vlen+N>mlen) vrealloc(vlen+N);
  loopj(N) add(T());
  for (int j=vlen-1;j>=I+N;j--) buf[j]=buf[j-N];
  loopj(N) buf[I+j]=P[j];
  return(&buf[I]);
 }
 // zmazanie N prvkov vektora od pozicie I
 void remove(int I,int N) {
  for (int j=I+N;j<vlen;j++) buf[j-N]=buf[j];
  vlen-=N;
 }
 // zmazanie prvku vektora z pozicie I
 T remove(int I) {
  T p=buf[I];
  for (int j=I+1;j<vlen;j++) buf[j-1]=buf[j];
  vlen--;
  return(p);
 }
 // zmazanie urcenych prvkov vektora
 void removeobj(const T &O) {
  loopi(vlen) if (buf[i]==O) remove(i--);
 }
 // ak je pocet prvkov vektora nulovy, vymenia sa parametre vektorov
 // v opacnom pripade sa prvky vektora V pripoja k prvkom this vektora
 void move(vector<T> &V) {
  if (!vlen) {
   swap(buf,V.buf); swap(vlen,V.vlen); swap(mlen,V.mlen);
  }
  else {
   vrealloc(vlen+V.vlen);
   if (V.vlen) memcpy(&buf[vlen],V.buf,V.vlen*sizeof(T));
   vlen+=V.vlen;
   V.vlen=0;
  }
 }
 // najdenie urceneho prvku vektora
 template <class U>
 int find(const U &O) {
  loopi(vlen) if (buf[i]==O) return(i);
  return(-1);
 }
};

#endif
