#ifndef _INCLUDES_H_
#define _INCLUDES_H_

// standardne hlavickove subory pre vsetky moduly **************************

// systemove kniznice ******************************************************
#include <stdlib.h>
#include <math.h>
#include <new>

// definicie pre Windows a Linux *******************************************
#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#define PATHDIV '\\'
#else
#define LINUX
#define PATHDIV '/'
#endif

// definicie pre aplikaciu *************************************************

// maximalny pocet fps
#define MAXFPS 200

// SDL kniznica ************************************************************
// /usr/lib/libSDL.so
// /usr/lib/libSDL_image.so
#include <SDL.h>
#include <SDL_image.h>

// OpenGL, GLU a GLEXT kniznice ********************************************
// vyber konkretnych kniznic je zavisly na pouzitom grafickom hardveri
// a implementacii OpenGL (napr. pomocou Mesa kniznice)
// /usr/lib/libGL.so, alebo v zavislosti na GPU
// napr. /usr/lib/nvidia/libGL.so
// /usr/lib/libGLU.so
// v pripade pouzitia GLEXT kniznice sa pouzije subor suvisiaci s GPU
// napr. /usr/lib/xorg/modules/extensions/nvidia/libglx.so
#define GL_GLEXT_LEGACY
// informacia pre gl.h aby nepripojil obsah suboru glext.h
#define __glext_h__
// pripoji obsah suborov gl.h a glu.h, nepripoji obsah suboru SDL_opengl.h
#define NO_SDL_GLEXT
#include <SDL_opengl.h>
#undef __glext_h__
#include <GL/glext.h>

// rozne pomocne nastroje **************************************************
#include "common/tools.h"
// rozne geometricke a matematicke funkcie *********************************
#include "common/math3d.h"
// praca s texturami *******************************************************
#include "common/texture.h"
// praca so shadermi *******************************************************
#include "common/shader.h"
// nacitanie MD3 suborov ***************************************************
#include "common/loadmd3.h"
// system castic ***********************************************************
#include "common/psystem.h"

// trojuholnik *************************************************************
struct _tri {
 _tri() { }  // konstruktor
 ~_tri() { } // destruktor

 GLuint id; // identifikator trojuholnika

 // typ trojuholnika
 // 0 - obycajny
 // 1 - zrkadlo
 unsigned char type;

 vec4 c; // farba

 vec3 n; // normala
 GLfloat d; // prvok D rovnice roviny, ktoru tvori trojuholnik,
            // prvkami A, B, C su prvky normaly trojuholnika

 vec2 t[3];     // texturovacie suradnice
 GLuint tex1id; // identifikator 1. OpenGL textury
 GLuint tex2id; // identifikator 2. OpenGL textury

 vec2 l[3];   // texturovacie suradnice pre lightmapu
 GLuint lmid; // identifikator lightmapy

 vec3 v[3];  // suradnice vrcholov
 vec3 vn[3]; // normaly vo vrcholoch

 vec4 cn[3]; // prvky A, B, C "koliznych normal" a prvok D,
             // ktore spolu definuju rovnice rovin pre jednotlive
             // strany trojuholnika - vyuzivaju sa pre urcenie, ci
             // dany 3D bod patri/nepatri trojuholniku

 vec3 tangent,binormal; // T a B pre tangent space

 // minimalne a maximalne suradnice vrcholov na danych osiach
 // okolo trojuholnika sa tak vytvori pomyselny hranol
 vec2 minmaxx,minmaxy,minmaxz;

 // indikator spracovania, ak done = true trojuholnik
 // bol spracovany a nebude sa dalej spracovavat
 bool done;
};

// svetelny zdroj **********************************************************
struct _light {
 _light() { }  // konstruktor
 ~_light() { } // destruktor

 GLuint id; // identifikator svetelneho zdroja

 vec4 c;    // farba

 vec3 p;    // poloha
};

// zrkadlo *****************************************************************
struct _mirror {
 _mirror() { }  // konstruktor
 ~_mirror() { } // destruktor

 GLuint id; // identifikator zrkadla

 GLuint tid; // identifikator 1. trojuholnika, ktory patri objektu zrkadla
 vec3 v[6];  // vrcholy dvoch trojuholnikov, ktore patria objektu zrkadla

 GLdouble cp_eq[4]; // rovnica orezavacej roviny

 // zoznam ukazovatelov na odrazene trojuholniky
 vector<_tri *> mtris;
};

// rebrik ******************************************************************
struct _ladder {
 _ladder() { minx=99999.9f; maxx=-99999.9f;
             miny=99999.9f; maxy=-99999.9f;
             minz=99999.9f; maxz=-99999.9f; }  // konstruktor
 ~_ladder() { } // destruktor

 // Avatar sa nachadza v blizkosti rebrika
 bool at_ladder(vec3 &AP) {
  v=-AP;
  if (v.x<minx || v.x>maxx ||
      v.y<miny || v.y>maxy ||
      v.z<minz || v.z>maxz) return(0);
  // nemusi sa robit pokial avatarWidth = rozmery aktivneho objemu rebrika
  /*
  d[0]=v.x-minx; d[1]=maxx-v.x;
  d[2]=v.y-miny; d[3]=maxy-v.y;
  d[4]=v.z-minz; d[5]=maxz-v.z;
  dx=min(d[0],d[1]); dy=min(d[2],d[3]); dz=min(d[4],d[5]);
  dxyz=min3(dx,dy,dz);
  if (dxyz==d[0]) { v.x-=dxyz; AP=-v; return(1); }
  if (dxyz==d[1]) { v.x+=dxyz; AP=-v; return(1); }
  if (dxyz==d[2]) { v.y-=dxyz; AP=-v; return(1); }
  if (dxyz==d[3]) { v.y+=dxyz; AP=-v; return(1); }
  if (dxyz==d[4]) { v.z-=dxyz; AP=-v; return(1); }
  if (dxyz==d[5]) { v.z+=dxyz; AP=-v; return(1); }
  */
  return(1);
 }

 // vykreslenie aktivneho objemu rebrika
 void draw(void) {
  glLineWidth(2.0f);
  glBegin(GL_LINES);
   glVertex3f(minx,miny,minz); glVertex3f(maxx,miny,minz);
   glVertex3f(maxx,miny,minz); glVertex3f(maxx,miny,maxz);
   glVertex3f(maxx,miny,maxz); glVertex3f(minx,miny,maxz);
   glVertex3f(minx,miny,maxz); glVertex3f(minx,miny,minz);

   glVertex3f(minx,maxy,minz); glVertex3f(maxx,maxy,minz);
   glVertex3f(maxx,maxy,minz); glVertex3f(maxx,maxy,maxz);
   glVertex3f(maxx,maxy,maxz); glVertex3f(minx,maxy,maxz);
   glVertex3f(minx,maxy,maxz); glVertex3f(minx,maxy,minz);

   glVertex3f(minx,miny,minz); glVertex3f(minx,maxy,minz);
   glVertex3f(maxx,miny,minz); glVertex3f(maxx,maxy,minz);
   glVertex3f(minx,miny,maxz); glVertex3f(minx,maxy,maxz);
   glVertex3f(maxx,miny,maxz); glVertex3f(maxx,maxy,maxz);
  glEnd();
  glLineWidth(1.0f);
 }

 // suradnice hranic aktivneho objemu
 GLfloat minx,maxx,miny,maxy,minz,maxz;
 vec3 v;                     // pomocny vektor
 GLfloat d[6],dx,dy,dz,dxyz; // pomocne premenne
};

// globalne premenne *******************************************************

// definovane v engine.cpp:
extern int scr_w,scr_w2;
extern int scr_h,scr_h2;
extern int fsaa;

// definovane v interface.cpp:
extern GLfloat difTime;
extern CTexture texture;
extern GLuint tid[MAX_TEX_NUM];
extern GLuint lmid[MAX_LM_NUM];
extern CMath3d m3d;
extern mat16 avamat;
extern bool glextMT,glextVP,glextFP,glextGLSL;
extern PFNGLACTIVETEXTUREARBPROC            glActiveTexture_;
extern PFNGLMULTITEXCOORD2FARBPROC          glMultiTexCoord2f_;
extern PFNGLGENPROGRAMSARBPROC              glGenPrograms_;
extern PFNGLDELETEPROGRAMSARBPROC           glDeletePrograms_;
extern PFNGLBINDPROGRAMARBPROC              glBindProgram_;
extern PFNGLPROGRAMSTRINGARBPROC            glProgramString_;
extern PFNGLPROGRAMENVPARAMETER4FVARBPROC   glProgramEnvParameter4fv_;
extern PFNGLPROGRAMLOCALPARAMETER4FVARBPROC glProgramLocalParameter4fv_;
extern PFNGLENABLEVERTEXATTRIBARRAYARBPROC  glEnableVertexAttribArray_;
extern PFNGLDISABLEVERTEXATTRIBARRAYARBPROC glDisableVertexAttribArray_;
extern PFNGLVERTEXATTRIBPOINTERARBPROC      glVertexAttribPointer_;
extern PFNGLCREATEPROGRAMOBJECTARBPROC      glCreateProgramObject_;
extern PFNGLDELETEOBJECTARBPROC             glDeleteObject_;
extern PFNGLUSEPROGRAMOBJECTARBPROC         glUseProgramObject_;
extern PFNGLCREATESHADEROBJECTARBPROC       glCreateShaderObject_;
extern PFNGLSHADERSOURCEARBPROC             glShaderSource_;
extern PFNGLCOMPILESHADERARBPROC            glCompileShader_;
extern PFNGLGETOBJECTPARAMETERIVARBPROC     glGetObjectParameteriv_;
extern PFNGLATTACHOBJECTARBPROC             glAttachObject_;
extern PFNGLLINKPROGRAMARBPROC              glLinkProgram_;
extern PFNGLGETUNIFORMLOCATIONARBPROC       glGetUniformLocation_;
extern PFNGLUNIFORM1IARBPROC                glUniform1i_;
extern PFNGLUNIFORM4FARBPROC                glUniform4f_;
extern PFNGLUNIFORMMATRIX4FVARBPROC         glUniformMatrix4fv_;

// definovane v backend.cpp:
extern CLoadMD3 *weapon;
extern vector<_tri> tris;
extern vector<_light> lights;
extern vector<_mirror> mirrors;
extern vector<_ladder> ladders;

// definovane v avatar.cpp:
extern vec3 Vup;

// definovane v shader.cpp:
extern CShader fl_vp;
extern CShader fl_fp;
extern CShader nm_fl_vp;
extern CShader nm_fl_fp;
extern CShader decal_vp;
extern CShader decal_fp;
extern CShader glsl_shader;

// definovane v loadmd3.cpp:
extern int stopaction;

#endif
